@@ -7,7 +7,7 @@
 #          Colin Blackburn <colb@bgs.ac.uk>
 #          Ricardo Garcia Silva <ricardo.garcia.silva@geobeyond.it>
 #
-# Copyright (c) 2024 Tom Kralidis
+# Copyright (c) 2025 Tom Kralidis
 # Copyright (c) 2022 Francesco Bartoli
 # Copyright (c) 2022 John A Stevenson and Colin Blackburn
 # Copyright (c) 2023 Ricardo Garcia Silva
@@ -40,7 +40,7 @@ Root level code of pygeoapi, parsing content provided by web framework.
 Returns content from plugins and sets responses.
 """
 
-from collections import OrderedDict
+from collections import ChainMap, OrderedDict
 from copy import deepcopy
 from datetime import datetime
 from functools import partial
@@ -64,8 +64,8 @@ from pygeoapi.provider.base import (
 
 from pygeoapi.util import (
     CrsTransformSpec, TEMPLATES, UrlPrefetcher, dategetter,
-    filter_dict_by_key_value, get_api_rules, get_base_url,
-    get_provider_by_type, get_provider_default, get_typed_value,
+    filter_dict_by_key_value, filter_providers_by_type, get_api_rules,
+    get_base_url, get_provider_by_type, get_provider_default, get_typed_value,
     get_crs_from_uri, get_supported_crs_list, render_j2_template, to_json
 )
 
@@ -595,7 +595,8 @@ class API:
         if format_ == F_HTML:
             headers['Content-Type'] = FORMAT_TYPES[F_HTML]
             content = render_j2_template(
-                self.config, 'exception.html', exception, SYSTEM_LOCALE)
+                self.tpl_config, self.config['server']['templates'],
+                'exception.html', exception, SYSTEM_LOCALE)
         else:
             content = to_json(exception, self.pretty_print)
 
@@ -619,12 +620,13 @@ class API:
             HTTPStatus.BAD_REQUEST, headers,
             request.format, 'InvalidParameterValue', msg)
 
-    def get_collections_url(self):
+    def get_collections_url(self) -> str:
         return f"{self.base_url}/collections"
 
-    def set_dataset_templates(self, dataset):
-        if 'templates' in self.config['resources'][dataset]:
-           self.tpl_config['server']['templates'] = self.config['resources'][dataset]['templates']  # noqa
+    def get_dataset_templates(self, dataset) -> dict:
+        templates = self.config['resources'][dataset].get('templates')
+
+        return templates or self.tpl_config['server']['templates']
 
     @staticmethod
     def _create_crs_transform_spec(
@@ -813,24 +815,23 @@ def landing_page(api: API,
     headers = request.get_response_headers(**api.api_headers)
     if request.format == F_HTML:  # render
 
-        fcm['processes'] = False
-        fcm['stac'] = False
-        fcm['collection'] = False
+        for resource_type in ['collection', 'process', 'stac-collection']:
+            fcm[resource_type] = False
 
-        if filter_dict_by_key_value(api.config['resources'],
-                                    'type', 'process'):
-            fcm['processes'] = True
+            found = filter_dict_by_key_value(api.config['resources'],
+                                             'type', resource_type)
+            if found:
+                fcm[resource_type] = True
+                if resource_type == 'collection':  # check for tiles
+                    for key, value in found.items():
+                        if filter_providers_by_type(value['providers'],
+                                                    'tile'):
+                            fcm['tile'] = True
 
-        if filter_dict_by_key_value(api.config['resources'],
-                                    'type', 'stac-collection'):
-            fcm['stac'] = True
+        content = render_j2_template(
+            api.tpl_config, api.config['server']['templates'],
+            'landing_page.html', fcm, request.locale)
 
-        if filter_dict_by_key_value(api.config['resources'],
-                                    'type', 'collection'):
-            fcm['collection'] = True
-
-        content = render_j2_template(api.tpl_config, 'landing_page.html',
-                                     fcm, request.locale)
         return headers, HTTPStatus.OK, content
 
     if request.format == F_JSONLD:
@@ -860,8 +861,10 @@ def openapi_(api: API, request: APIRequest) -> Tuple[dict, int, str]:
         data = {
             'openapi-document-path': path
         }
-        content = render_j2_template(api.tpl_config, template, data,
-                                     request.locale)
+        content = render_j2_template(
+            api.tpl_config, api.config['server']['templates'], template, data,
+            request.locale)
+
         return headers, HTTPStatus.OK, content
 
     headers['Content-Type'] = 'application/vnd.oai.openapi+json;version=3.0'  # noqa
@@ -908,8 +911,10 @@ def conformance(api, request: APIRequest) -> Tuple[dict, int, str]:
 
     headers = request.get_response_headers(**api.api_headers)
     if request.format == F_HTML:  # render
-        content = render_j2_template(api.tpl_config, 'conformance.html',
-                                     conformance, request.locale)
+        content = render_j2_template(
+            api.tpl_config, api.config['server']['templates'],
+            'conformance.html', conformance, request.locale)
+
         return headers, HTTPStatus.OK, content
 
     return headers, HTTPStatus.OK, to_json(conformance, api.pretty_print)
@@ -1064,9 +1069,23 @@ def describe_collections(api: API, request: APIRequest,
             'href': f'{api.get_collections_url()}/{k}?f={F_HTML}'
         })
 
-        if collection_data_type in ['feature', 'coverage', 'record']:
+        if collection_data_type == 'record':
             collection['links'].append({
                 'type': FORMAT_TYPES[F_JSON],
+                'rel': 'http://www.opengis.net/def/rel/ogc/1.0/ogc-catalog',
+                'title': l10n.translate('Record catalogue as JSON', request.locale),  # noqa
+                'href': f'{api.get_collections_url()}/{k}?f={F_JSON}'
+            })
+            collection['links'].append({
+                'type': FORMAT_TYPES[F_HTML],
+                'rel': 'http://www.opengis.net/def/rel/ogc/1.0/ogc-catalog',
+                'title': l10n.translate('Record catalogue as HTML', request.locale),  # noqa
+                'href': f'{api.get_collections_url()}/{k}?f={F_HTML}'
+            })
+
+        if collection_data_type in ['feature', 'coverage', 'record']:
+            collection['links'].append({
+                'type': 'application/schema+json',
                 'rel': f'{OGC_RELTYPES_BASE}/schema',
                 'title': l10n.translate('Schema of collection in JSON', request.locale),  # noqa
                 'href': f'{api.get_collections_url()}/{k}/schema?f={F_JSON}'  # noqa
@@ -1113,12 +1132,12 @@ def describe_collections(api: API, request: APIRequest,
                 'href': f'{api.get_collections_url()}/{k}/items?f={F_HTML}'  # noqa
             })
 
-            # OAPIF Part 2 - list supported CRSs and StorageCRS
-            if collection_data_type == 'feature':
-                collection['crs'] = get_supported_crs_list(collection_data, DEFAULT_CRS_LIST) # noqa
-                collection['storageCRS'] = collection_data.get('storage_crs', DEFAULT_STORAGE_CRS) # noqa
-                if 'storage_crs_coordinate_epoch' in collection_data:
-                    collection['storageCrsCoordinateEpoch'] = collection_data.get('storage_crs_coordinate_epoch') # noqa
+        # OAPIF Part 2 - list supported CRSs and StorageCRS
+        if collection_data_type in ['edr', 'feature']:
+            collection['crs'] = get_supported_crs_list(collection_data, DEFAULT_CRS_LIST) # noqa
+            collection['storageCRS'] = collection_data.get('storage_crs', DEFAULT_STORAGE_CRS) # noqa
+            if 'storage_crs_coordinate_epoch' in collection_data:
+                collection['storageCrsCoordinateEpoch'] = collection_data.get('storage_crs_coordinate_epoch') # noqa
 
         elif collection_data_type == 'coverage':
             # TODO: translate
@@ -1161,6 +1180,16 @@ def describe_collections(api: API, request: APIRequest,
                         'cellsCount': p._coverage_properties['height'],
                         'resolution': p._coverage_properties['resy']
                     }]
+                    if 'time_range' in p._coverage_properties:
+                        collection['extent']['temporal'] = {
+                            'interval': [p._coverage_properties['time_range']]
+                        }
+                        if 'restime' in p._coverage_properties:
+                            collection['extent']['temporal']['grid'] = {
+                                'resolution': p._coverage_properties['restime']  # noqa
+                            }
+                    if 'uad' in p._coverage_properties:
+                        collection['extent'].update(p._coverage_properties['uad'])  # noqa
 
         try:
             tile = get_provider_by_type(v['providers'], 'tile')
@@ -1235,6 +1264,12 @@ def describe_collections(api: API, request: APIRequest,
                         'id': key,
                         'type': 'Parameter',
                         'name': value['title'],
+                        'observedProperty': {
+                            'label': {
+                                'id': key,
+                                'en': value['title']
+                            },
+                        },
                         'unit': {
                             'label': {
                                 'en': value['title']
@@ -1250,8 +1285,11 @@ def describe_collections(api: API, request: APIRequest,
                 data_query = {
                     'link': {
                         'href': f'{api.get_collections_url()}/{k}/{qt}',
-                        'rel': 'data'
-                     }
+                        'rel': 'data',
+                        'variables': {
+                            'query_type': qt
+                        }
+                    }
                 }
                 collection['data_queries'][qt] = data_query
 
@@ -1264,13 +1302,13 @@ def describe_collections(api: API, request: APIRequest,
                     'type': 'application/json',
                     'rel': 'data',
                     'title': title1,
-                    'href': f'{api.get_collections_url()}/{k}/{qt}?f={F_JSON}'  # noqa
+                    'href': f'{api.get_collections_url()}/{k}/{qt}?f={F_JSON}'
                 })
                 collection['links'].append({
                     'type': FORMAT_TYPES[F_HTML],
                     'rel': 'data',
                     'title': title2,
-                    'href': f'{api.get_collections_url()}/{k}/{qt}?f={F_HTML}'  # noqa
+                    'href': f'{api.get_collections_url()}/{k}/{qt}?f={F_HTML}'
                 })
 
         if dataset is not None and k == dataset:
@@ -1303,14 +1341,14 @@ def describe_collections(api: API, request: APIRequest,
     if request.format == F_HTML:  # render
         fcm['collections_path'] = api.get_collections_url()
         if dataset is not None:
-            api.set_dataset_templates(dataset)
-            content = render_j2_template(api.tpl_config,
+            tpl_config = api.get_dataset_templates(dataset)
+            content = render_j2_template(api.tpl_config, tpl_config,
                                          'collections/collection.html',
                                          fcm, request.locale)
         else:
-            content = render_j2_template(api.tpl_config,
-                                         'collections/index.html',
-                                         fcm, request.locale)
+            content = render_j2_template(
+                api.tpl_config, api.config['server']['templates'],
+                'collections/index.html', fcm, request.locale)
 
         return headers, HTTPStatus.OK, content
 
@@ -1380,7 +1418,7 @@ def get_collection_schema(api: API, request: Union[APIRequest, Any],
 
     if p.type != 'coverage':
         schema['properties']['geometry'] = {
-            '$ref': 'https://geojson.org/schema/Geometry.json',
+            'format': 'geometry-any',
             'x-ogc-role': 'primary-geometry'
         }
 
@@ -1395,14 +1433,14 @@ def get_collection_schema(api: API, request: Union[APIRequest, Any],
             schema['properties'][k]['x-ogc-role'] = 'primary-instant'
 
     if request.format == F_HTML:  # render
-        api.set_dataset_templates(dataset)
+        tpl_config = api.get_dataset_templates(dataset)
         schema['title'] = l10n.translate(
             api.config['resources'][dataset]['title'], request.locale)
 
         schema['collections_path'] = api.get_collections_url()
         schema['dataset_path'] = f'{api.get_collections_url()}/{dataset}'
 
-        content = render_j2_template(api.tpl_config,
+        content = render_j2_template(api.tpl_config, tpl_config,
                                      'collections/schema.html',
                                      schema, request.locale)
 
@@ -1590,3 +1628,43 @@ def validate_subset(value: str) -> dict:
         subsets[subset_name] = list(map(get_typed_value, values))
 
     return subsets
+
+
+def evaluate_limit(requested: Union[None, int], server_limits: dict,
+                   collection_limits: dict) -> int:
+    """
+    Helper function to evaluate limit parameter
+
+    :param requested: the limit requested by the client
+    :param server_limits: `dict` of server limits
+    :param collection_limits: `dict` of collection limits
+
+    :returns: `int` of evaluated limit
+    """
+
+    effective_limits = ChainMap(collection_limits, server_limits)
+
+    default = effective_limits.get('default_items', 10)
+    max_ = effective_limits.get('max_items', 10)
+    on_exceed = effective_limits.get('on_exceed', 'throttle')
+
+    LOGGER.debug(f'Requested limit: {requested}')
+    LOGGER.debug(f'Default limit: {default}')
+    LOGGER.debug(f'Maximum limit: {max_}')
+    LOGGER.debug(f'On exceed: {on_exceed}')
+
+    if requested is None:
+        LOGGER.debug('no limit requested; returning default')
+        return default
+
+    requested2 = get_typed_value(requested)
+    if not isinstance(requested2, int):
+        raise ValueError('limit value should be an integer')
+
+    if requested2 <= 0:
+        raise ValueError('limit value should be strictly positive')
+    elif requested2 > max_ and on_exceed == 'error':
+        raise RuntimeError('Limit exceeded; throwing errror')
+    else:
+        LOGGER.debug('limit requested')
+        return min(requested2, max_)
