@@ -49,6 +49,7 @@ from pygeoapi.models.provider.base import (TilesMetadataFormat,
 from pygeoapi.provider.base import (
     ProviderGenericError, ProviderTypeError
 )
+from pygeoapi.provider.tile import ProviderTileNotFoundError
 
 from pygeoapi.util import (
     get_provider_by_type, to_json, filter_dict_by_key_value,
@@ -141,12 +142,17 @@ def get_collection_tiles(api: API, request: APIRequest,
 
     tiling_schemes = p.get_tiling_schemes()
 
+    datatype = 'vector'
+
+    if t['format']['mimetype'].startswith('image'):
+        datatype = 'map'
+
     for matrix in tiling_schemes:
         tile_matrix = {
             'title': dataset,
             'tileMatrixSetURI': matrix.tileMatrixSetURI,
             'crs': matrix.crs,
-            'dataType': 'vector',
+            'dataType': datatype,
             'links': []
         }
         tile_matrix['links'].append({
@@ -171,6 +177,7 @@ def get_collection_tiles(api: API, request: APIRequest,
         tiles['tilesets'].append(tile_matrix)
 
     if request.format == F_HTML:  # render
+        tpl_config = api.get_dataset_templates(dataset)
         tiles['id'] = dataset
         tiles['title'] = l10n.translate(
             api.config['resources'][dataset]['title'], SYSTEM_LOCALE)
@@ -183,7 +190,7 @@ def get_collection_tiles(api: API, request: APIRequest,
         tiles['collections_path'] = api.get_collections_url()
         tiles['tile_type'] = p.tile_type
 
-        content = render_j2_template(api.tpl_config,
+        content = render_j2_template(api.tpl_config, tpl_config,
                                      'collections/tiles/index.html', tiles,
                                      request.locale)
 
@@ -192,7 +199,6 @@ def get_collection_tiles(api: API, request: APIRequest,
     return headers, HTTPStatus.OK, to_json(tiles, api.pretty_print)
 
 
-# TODO: no test for this function?
 def get_collection_tiles_data(
         api: API, request: APIRequest,
         dataset=None, matrix_id=None,
@@ -240,7 +246,7 @@ def get_collection_tiles_data(
         if content is None:
             msg = 'identifier not found'
             return api.get_exception(
-                HTTPStatus.NOT_FOUND, headers, format_, 'NotFound', msg)
+                HTTPStatus.NO_CONTENT, headers, format_, 'NocContent', msg)
         else:
             return headers, HTTPStatus.OK, content
 
@@ -250,13 +256,16 @@ def get_collection_tiles_data(
         return api.get_exception(
             HTTPStatus.BAD_REQUEST, headers, format_,
             'InvalidParameterValue', msg)
+    except ProviderTileNotFoundError:
+        msg = 'Tile not found'
+        LOGGER.info(msg)
+        return headers, HTTPStatus.NOT_FOUND, msg
     except ProviderGenericError as err:
         return api.get_exception(
             err.http_status_code, headers, request.format,
             err.ogc_exception_code, err.message)
 
 
-# TODO: no test for this function?
 def get_collection_tiles_metadata(
     api: API, request: APIRequest,
         dataset=None, matrix_id=None) -> Tuple[dict, int, str]:
@@ -317,13 +326,15 @@ def get_collection_tiles_metadata(
         language=prv_locale)
 
     if request.format == F_HTML:  # render
-        content = render_j2_template(api.tpl_config,
+        tpl_config = api.get_dataset_templates(dataset)
+        content = render_j2_template(api.tpl_config, tpl_config,
                                      'collections/tiles/metadata.html',
                                      tiles_metadata, request.locale)
 
-        return headers, HTTPStatus.OK, content
     else:
-        return headers, HTTPStatus.OK, tiles_metadata
+        content = to_json(tiles_metadata, api.pretty_print)
+
+    return headers, HTTPStatus.OK, content
 
 
 def tilematrixsets(api: API,
@@ -377,7 +388,7 @@ def tilematrixsets(api: API,
     }]
 
     if request.format == F_HTML:  # render
-        content = render_j2_template(api.tpl_config,
+        content = render_j2_template(api.tpl_config, api.tpl_config,
                                      'tilematrixsets/index.html',
                                      tms, request.locale)
         return headers, HTTPStatus.OK, content
@@ -425,14 +436,13 @@ def tilematrixset(api: API,
     }
 
     if request.format == F_HTML:  # render
-        content = render_j2_template(api.tpl_config,
+        content = render_j2_template(api.tpl_config, api.tpl_config,
                                      'tilematrixsets/tilematrixset.html',
                                      tms, request.locale)
         return headers, HTTPStatus.OK, content
 
     return headers, HTTPStatus.OK, to_json(tms, api.pretty_print)
 
-
 def get_oas_30(cfg: dict, locale: str) -> tuple[list[dict[str, str]], dict[str, dict]]:  # noqa
     """
     Get OpenAPI fragments
@@ -462,12 +472,17 @@ def get_oas_30(cfg: dict, locale: str) -> tuple[list[dict[str, str]], dict[str,
             title = l10n.translate(v['title'], locale)
             description = l10n.translate(v['description'], locale)
 
+            datatype = 'vector'
+
+            if tile_extension['format']['mimetype'].startswith('image'):
+                datatype = 'map'
+
             paths[tiles_path] = {
                 'get': {
                     'summary': f'Fetch a {title} tiles description',
                     'description': description,
                     'tags': [k],
-                    'operationId': f'describe{k.capitalize()}Tiles',
+                    'operationId': f'describe{k.capitalize()}.collection.{datatype}.getTileSetsList',  # noqa
                     'parameters': [
                         {'$ref': '#/components/parameters/f'},
                         {'$ref': '#/components/parameters/lang'}
@@ -488,7 +503,7 @@ def get_oas_30(cfg: dict, locale: str) -> tuple[list[dict[str, str]], dict[str,
                     'summary': f'Get a {title} tile',
                     'description': description,
                     'tags': [k],
-                    'operationId': f'get{k.capitalize()}Tiles',
+                    'operationId': f'get{k.capitalize()}.collection.{datatype}.getTile',  # noqa
                     'parameters': [
                         {'$ref': f"{OPENAPI_YAML['oapit']}#/components/parameters/tileMatrixSetId"}, # noqa
                         {'$ref': f"{OPENAPI_YAML['oapit']}#/components/parameters/tileMatrix"},  # noqa
