@@ -73,7 +73,12 @@ BLUEPRINT = Blueprint(
     static_folder=STATIC_FOLDER,
     url_prefix=API_RULES.get_url_prefix('flask')
 )
-ADMIN_BLUEPRINT = Blueprint('admin', __name__, static_folder=STATIC_FOLDER)
+ADMIN_BLUEPRINT = Blueprint(
+    'admin',
+    __name__,
+    static_folder=STATIC_FOLDER,
+    url_prefix=API_RULES.get_url_prefix('flask')
+)
 
 # CORS: optionally enable from config.
 if CONFIG['server'].get('cors', False):
@@ -281,7 +286,7 @@ def collection_items(collection_id, item_id=None):
                             skip_valid_check=True)
                 else:
                     return execute_from_flask(
-                            itemtypes_api.post_collection_items, request,
+                            itemtypes_api.get_collection_items, request,
                             collection_id, skip_valid_check=True)
         elif request.method == 'OPTIONS':
             return execute_from_flask(
@@ -465,8 +470,8 @@ def get_job_result(job_id=None):
 @BLUEPRINT.route('/collections/<path:collection_id>/radius')
 @BLUEPRINT.route('/collections/<path:collection_id>/trajectory')
 @BLUEPRINT.route('/collections/<path:collection_id>/corridor')
-@BLUEPRINT.route('/collections/<path:collection_id>/locations/<location_id>')  # noqa
-@BLUEPRINT.route('/collections/<path:collection_id>/locations')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/locations/<location_id>')
+@BLUEPRINT.route('/collections/<path:collection_id>/locations')
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/position')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/area')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/cube')  # noqa
@@ -475,6 +480,8 @@ def get_job_result(job_id=None):
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/corridor')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/locations/<location_id>')  # noqa
 @BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>/locations')  # noqa
+@BLUEPRINT.route('/collections/<path:collection_id>/instances/<instance_id>')
+@BLUEPRINT.route('/collections/<path:collection_id>/instances')
 def get_collection_edr_query(collection_id, instance_id=None,
                              location_id=None):
     """
@@ -487,6 +494,14 @@ def get_collection_edr_query(collection_id, instance_id=None,
     :returns: HTTP response
     """
 
+    if (request.path.endswith('instances') or
+            (instance_id is not None and
+             request.path.endswith(instance_id))):
+        return execute_from_flask(
+            edr_api.get_collection_edr_instances, request, collection_id,
+            instance_id
+        )
+
     if location_id:
         query_type = 'locations'
     else:
@@ -494,8 +509,7 @@ def get_collection_edr_query(collection_id, instance_id=None,
 
     return execute_from_flask(
         edr_api.get_collection_edr_query, request, collection_id, instance_id,
-        query_type, location_id,
-        skip_valid_check=True,
+        query_type, location_id, skip_valid_check=True
     )
 
 
