@@ -34,7 +34,7 @@ Returns content as linked data representations
 import logging
 from typing import Callable
 
-from pygeoapi.util import is_url
+from pygeoapi.util import is_url, render_j2_template, url_join
 from pygeoapi import l10n
 from shapely.geometry import shape
 from shapely.ops import unary_union
@@ -138,7 +138,7 @@ def jsonldify_collection(cls, collection: dict, locale_: str) -> dict:
 
     dataset = {
         "@type": "Dataset",
-        "@id": f"{cls.config['server']['url']}/collections/{collection['id']}",
+        "@id": f"{cls.base_url}/collections/{collection['id']}",
         "name": l10n.translate(collection['title'], locale_),
         "description": l10n.translate(collection['description'], locale_),
         "license": cls.fcmld['license'],
@@ -172,14 +172,14 @@ def jsonldify_collection(cls, collection: dict, locale_: str) -> dict:
     return dataset
 
 
-def geojson2jsonld(config: dict, data: dict, dataset: str,
+def geojson2jsonld(cls, data: dict, dataset: str,
                    identifier: str = None, id_field: str = 'id') -> str:
     """
     Render GeoJSON-LD from a GeoJSON base. Inserts a @context that can be
     read from, and extended by, the pygeoapi configuration for a particular
     dataset.
 
-    :param config: dict of configuration
+    :param cls: API object
     :param data: dict of data:
     :param dataset: dataset identifier
     :param identifier: item identifier (optional)
@@ -188,26 +188,25 @@ def geojson2jsonld(config: dict, data: dict, dataset: str,
     :returns: string of rendered JSON (GeoJSON-LD)
     """
 
-    context = config['resources'][dataset].get('context', []).copy()
+    LOGGER.debug('Fetching context from resource configuration')
+    context = cls.config['resources'][dataset].get('context', []).copy()
+    templates = cls.get_dataset_templates(dataset)
+
     defaultVocabulary = {
         'schema': 'https://schema.org/',
-        id_field: '@id',
+        'gsp': 'http://www.opengis.net/ont/geosparql#',
         'type': '@type'
     }
 
     if identifier:
-        # Single jsonld
-        defaultVocabulary.update({
-            'geosparql': 'http://www.opengis.net/ont/geosparql#'
-        })
-
         # Expand properties block
         data.update(data.pop('properties'))
 
         # Include multiple geometry encodings
-        data['type'] = 'schema:Place'
-        jsonldify_geometry(data)
-        data[id_field] = identifier
+        if (data.get('geometry') is not None):
+            jsonldify_geometry(data)
+
+        data['@id'] = identifier
 
     else:
         # Collection of jsonld
@@ -216,18 +215,25 @@ def geojson2jsonld(config: dict, data: dict, dataset: str,
             'FeatureCollection': 'schema:itemList'
         })
 
-        data['@id'] = f"{config['server']['url']}/collections/{dataset}"
+        ds_url = url_join(cls.get_collections_url(), dataset)
+        data['@id'] = ds_url
 
         for i, feature in enumerate(data['features']):
             # Get URI for each feature
-            identifier = feature.get(id_field,
-                                     feature['properties'].get(id_field, ''))
-            if not is_url(str(identifier)):
-                identifier = f"config['server']['url']/collections/{dataset}/items/{feature['id']}"  # noqa
+            identifier_ = feature.get(id_field,
+                                      feature['properties'].get(id_field, ''))
+            if not is_url(str(identifier_)):
+                identifier_ = f"{ds_url}/items/{feature['id']}"  # noqa
+
+            # Include multiple geometry encodings
+            if feature.get('geometry') is not None:
+                jsonldify_geometry(feature)
 
             data['features'][i] = {
-                id_field: identifier,
-                'type': 'schema:Place'
+                '@id': identifier_,
+                'type': 'schema:Place',
+                **feature.pop('properties'),
+                **feature
             }
 
     if data.get('timeStamp', False):
@@ -240,7 +246,21 @@ def geojson2jsonld(config: dict, data: dict, dataset: str,
         **data
     }
 
-    return ldjsonData
+    if identifier:
+        # Render jsonld template for single item
+        LOGGER.debug('Rendering JSON-LD item template')
+        content = render_j2_template(
+            cls.tpl_config, templates,
+            'collections/items/item.jsonld', ldjsonData)
+
+    else:
+        # Render jsonld template for /items
+        LOGGER.debug('Rendering JSON-LD items template')
+        content = render_j2_template(
+            cls.tpl_config, templates,
+            'collections/items/index.jsonld', ldjsonData)
+
+    return content
 
 
 def jsonldify_geometry(feature: dict) -> None:
@@ -253,6 +273,8 @@ def jsonldify_geometry(feature: dict) -> None:
     :returns: None
     """
 
+    feature['type'] = 'schema:Place'
+
     geo = feature.get('geometry')
     geom = shape(geo)
 
@@ -260,16 +282,20 @@ def jsonldify_geometry(feature: dict) -> None:
     feature['geometry'] = feature.pop('geometry')
 
     # Geosparql geometry
-    feature['geosparql:hasGeometry'] = {
+    feature['gsp:hasGeometry'] = {
         '@type': f'http://www.opengis.net/ont/sf#{geom.geom_type}',
-        'geosparql:asWKT': {
+        'gsp:asWKT': {
             '@type': 'http://www.opengis.net/ont/geosparql#wktLiteral',
             '@value': f'{geom.wkt}'
         }
     }
 
     # Schema geometry
-    feature['schema:geo'] = geom2schemageo(geom)
+    try:
+        feature['schema:geo'] = geom2schemageo(geom)
+    except AttributeError:
+        msg = f'Unable to parse schema geometry for {feature["id"]}'
+        LOGGER.warning(msg)
 
 
 def geom2schemageo(geom: shape) -> dict:
@@ -289,23 +315,23 @@ def geom2schemageo(geom: shape) -> dict:
         }
 
     elif geom.geom_type == 'LineString':
-        _ = [f'{x},{y}' for (x, y) in geom.coords[:]]
-        f['schema:line'] = ' '.join(_)
+        points = [f'{x},{y}' for (x, y, *_) in geom.coords[:]]
+        f['schema:line'] = ' '.join(points)
         return f
 
     elif geom.geom_type == 'MultiLineString':
         points = list()
-        [points.extend(p.coords[:]) for p in geom.geoms]
-        _ = [f'{x},{y}' for (x, y) in points]
-        f['schema:line'] = ' '.join(_)
+        for line in geom.geoms:
+            points.extend([f'{x},{y}' for (x, y, *_) in line.coords[:]])
+        f['schema:line'] = ' '.join(points)
         return f
 
     elif geom.geom_type == 'MultiPoint':
-        poly_geom = [(p.x, p.y) for p in geom.geoms]
-        poly_geom.append(poly_geom[0])
+        points = [(x, y) for pt in geom.geoms for (x, y, *_) in pt.coords]
+        points.append(points[0])
 
     elif geom.geom_type == 'Polygon':
-        poly_geom = geom.exterior.coords[:]
+        points = geom.exterior.coords[:]
 
     elif geom.geom_type == 'MultiPolygon':
         # MultiPolygon to Polygon (buffer of 0 helps ensure manifold polygon)
@@ -314,20 +340,17 @@ def geom2schemageo(geom: shape) -> dict:
             LOGGER.debug(f'Invalid MultiPolygon: {poly.geom_type}')
             poly = poly.convex_hull
             LOGGER.debug(f'New MultiPolygon: {poly.geom_type}')
-        poly_geom = poly.exterior.coords[:]
+        points = poly.exterior.coords[:]
 
     else:
-        poly_geom = list()
+        points = list()
         for p in geom.geoms:
             try:
-                poly_geom.extend(p.coords[:])
+                points.extend(p.coords[:])
             except NotImplementedError:
-                poly_geom.extend(p.exterior.coords[:])
+                points.extend(p.exterior.coords[:])
 
-    try:
-        schema_polygon = [f'{x},{y}' for (x, y) in poly_geom]
-    except ValueError:
-        schema_polygon = [f'{x},{y},{z}' for (x, y, z) in poly_geom]
+    schema_polygon = [f'{x},{y}' for (x, y, *_) in points]
 
     f['schema:polygon'] = ' '.join(schema_polygon)
 
