@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2022 Tom Kralidis
+# Copyright (c) 2025 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -32,8 +32,9 @@ import csv
 import itertools
 import logging
 
-from pygeoapi.provider.base import (BaseProvider, ProviderQueryError,
-                                    ProviderItemNotFoundError)
+from pygeoapi.provider.base import (BaseProvider, ProviderInvalidQueryError,
+                                    ProviderItemNotFoundError,
+                                    ProviderQueryError)
 from pygeoapi.util import get_typed_value, crs_transform
 
 LOGGER = logging.getLogger(__name__)
@@ -54,7 +55,7 @@ class CSVProvider(BaseProvider):
         super().__init__(provider_def)
         self.geometry_x = provider_def['geometry']['x_field']
         self.geometry_y = provider_def['geometry']['y_field']
-        self.fields = self.get_fields()
+        self.get_fields()
 
     def get_fields(self):
         """
@@ -62,32 +63,31 @@ class CSVProvider(BaseProvider):
 
         :returns: dict of fields
         """
+        if not self._fields:
+            LOGGER.debug('Treating all columns as string types')
+            with open(self.data) as ff:
+                LOGGER.debug('Serializing DictReader')
+                data_ = csv.DictReader(ff)
 
-        LOGGER.debug('Treating all columns as string types')
-        with open(self.data) as ff:
-            LOGGER.debug('Serializing DictReader')
-            data_ = csv.DictReader(ff)
-            fields = {}
+                row = next(data_)
 
-            row = next(data_)
+                for key, value in row.items():
+                    LOGGER.debug(f'key: {key}, value: {value}')
+                    value2 = get_typed_value(value)
+                    if key in [self.geometry_x, self.geometry_y]:
+                        continue
+                    if key == self.id_field:
+                        type_ = 'string'
+                    elif isinstance(value2, float):
+                        type_ = 'number'
+                    elif isinstance(value2, int):
+                        type_ = 'integer'
+                    else:
+                        type_ = 'string'
 
-            for key, value in row.items():
-                LOGGER.debug(f'key: {key}, value: {value}')
-                value2 = get_typed_value(value)
-                if key in [self.geometry_x, self.geometry_y]:
-                    continue
-                if key == self.id_field:
-                    type_ = 'string'
-                elif isinstance(value2, float):
-                    type_ = 'number'
-                elif isinstance(value2, int):
-                    type_ = 'integer'
-                else:
-                    type_ = 'string'
+                    self._fields[key] = {'type': type_}
 
-                fields[key] = {'type': type_}
-
-            return fields
+        return self._fields
 
     def _load(self, offset=0, limit=10, resulttype='results',
               identifier=None, bbox=[], datetime_=None, properties=[],
@@ -121,6 +121,12 @@ class CSVProvider(BaseProvider):
             LOGGER.debug('Serializing DictReader')
             data_ = csv.DictReader(ff)
             if properties:
+                for prop in properties:
+                    if prop[0] not in data_.fieldnames:
+                        msg = 'Invalid fieldname'
+                        LOGGER.error(msg)
+                        raise ProviderInvalidQueryError(msg)
+
                 data_ = filter(
                     lambda p: all(
                         [p[prop[0]] == prop[1] for prop in properties]), data_)
@@ -130,16 +136,17 @@ class CSVProvider(BaseProvider):
                 feature_collection['numberMatched'] = len(list(data_))
                 return feature_collection
             LOGGER.debug('Slicing CSV rows')
-            for row in itertools.islice(data_, offset, offset+limit):
+            for row in itertools.islice(data_, 0, None):
                 try:
                     coordinates = [
                         float(row.pop(self.geometry_x)),
                         float(row.pop(self.geometry_y)),
                     ]
                 except ValueError:
-                    msg = f'Skipping row with invalid geometry: {row.get(self.id_field)}'  # noqa
-                    LOGGER.error(msg)
-                    continue
+                    msg = f'Row with invalid geometry: {row.get(self.id_field)}, setting coordinates to None'  # noqa
+                    LOGGER.warning(msg)
+                    coordinates = None
+
                 feature = {'type': 'Feature'}
                 feature['id'] = row.pop(self.id_field)
                 if not skip_geometry:
@@ -178,6 +185,9 @@ class CSVProvider(BaseProvider):
         elif identifier is not None and found:
             return result
 
+        features_returned = feature_collection['features'][offset:offset+limit]
+        feature_collection['features'] = features_returned
+
         feature_collection['numberReturned'] = len(
             feature_collection['features'])
 
