@@ -36,13 +36,14 @@ import logging
 import uuid
 
 from opensearchpy import OpenSearch, helpers
-from opensearch_dsl import Search, Q
+from opensearch_dsl import Search
+
+from pygeofilter.backends.opensearch import to_filter
 
 from pygeoapi.provider.base import (BaseProvider, ProviderConnectionError,
                                     ProviderQueryError,
                                     ProviderItemNotFoundError)
-from pygeoapi.models.cql import CQLModel, get_next_node
-from pygeoapi.util import get_envelope, crs_transform
+from pygeoapi.util import crs_transform
 
 
 LOGGER = logging.getLogger(__name__)
@@ -294,7 +295,7 @@ class OpenSearchProvider(BaseProvider):
         try:
             LOGGER.debug('querying OpenSearch')
             if filterq:
-                LOGGER.debug(f'adding cql object: {filterq.json()}')
+                LOGGER.debug(f'adding cql object: {filterq}')
                 query = update_query(input_query=query, cql=filterq)
             LOGGER.debug(json.dumps(query, indent=4))
 
@@ -560,183 +561,9 @@ class OpenSearchCatalogueProvider(OpenSearchProvider):
         return f'<OpenSearchCatalogueProvider> {self.data}'
 
 
-class OpenSearchQueryBuilder:
-    def __init__(self):
-        self._operation = None
-        self.must_value = {}
-        self.should_value = {}
-        self.mustnot_value = {}
-        self.filter_value = {}
-
-    def must(self, must_value):
-        self.must_value = must_value
-        return self
-
-    def should(self, should_value):
-        self.should_value = should_value
-        return self
-
-    def must_not(self, mustnot_value):
-        self.mustnot_value = mustnot_value
-        return self
-
-    def filter(self, filter_value):
-        self.filter_value = filter_value
-        return self
-
-    @property
-    def operation(self):
-        return self._operation
-
-    @operation.setter
-    def operation(self, value):
-        self._operation = value
-
-    def build(self):
-        if self.must_value:
-            must_clause = self.must_value or {}
-        if self.should_value:
-            should_clause = self.should_value or {}
-        if self.mustnot_value:
-            mustnot_clause = self.mustnot_value or {}
-        if self.filter_value:
-            filter_clause = self.filter_value or {}
-        else:
-            filter_clause = {}
-
-        # to figure out how to deal with logical operations
-        # return match_clause & range_clause
-        clauses = must_clause or should_clause or mustnot_clause
-        filters = filter_clause
-        if self.operation == 'and':
-            res = Q(
-                'bool',
-                must=[clause for clause in clauses],
-                filter=[filter for filter in filters])
-        elif self.operation == 'or':
-            res = Q(
-                'bool',
-                should=[clause for clause in clauses],
-                filter=[filter for filter in filters])
-        elif self.operation == 'not':
-            res = Q(
-                'bool',
-                must_not=[clause for clause in clauses],
-                filter=[filter for filter in filters])
-        else:
-            if filters:
-                res = Q(
-                    'bool',
-                    must=[clauses],
-                    filter=[filters])
-            else:
-                res = Q(
-                    'bool',
-                    must=[clauses])
-
-        return res
-
-
-def _build_query(q, cql):
-
-    # this would be handled by the AST with the traverse of CQL model
-    op, node = get_next_node(cql.__root__)
-    q.operation = op
-    if isinstance(node, list):
-        query_list = []
-        for elem in node:
-            op, next_node = get_next_node(elem)
-            if not getattr(next_node, 'between', 0) == 0:
-                property = next_node.between.value.__root__.__root__.property
-                lower = next_node.between.lower.__root__.__root__
-                upper = next_node.between.upper.__root__.__root__
-                query_list.append(Q(
-                    {
-                        'range':
-                            {
-                                f'{property}': {
-                                    'gte': lower, 'lte': upper
-                                }
-                            }
-                    }
-                ))
-            if not getattr(next_node, '__root__', 0) == 0:
-                scalars = tuple(next_node.__root__.eq.__root__)
-                property = scalars[0].__root__.property
-                value = scalars[1].__root__.__root__
-                query_list.append(Q(
-                    {'match': {f'{property}': f'{value}'}}
-                ))
-        q.must(query_list)
-    elif not getattr(node, 'between', 0) == 0:
-        property = node.between.value.__root__.__root__.property
-        lower = None
-        if not getattr(node.between.lower,
-                       '__root__', 0) == 0:
-            lower = node.between.lower.__root__.__root__
-        upper = None
-        if not getattr(node.between.upper,
-                       '__root__', 0) == 0:
-            upper = node.between.upper.__root__.__root__
-        query = Q(
-            {
-                'range':
-                    {
-                        f'{property}': {
-                            'gte': lower, 'lte': upper
-                        }
-                    }
-            }
-        )
-        q.must(query)
-    elif not getattr(node, '__root__', 0) == 0:
-        next_op, next_node = get_next_node(node)
-        if not getattr(next_node, 'eq', 0) == 0:
-            scalars = tuple(next_node.eq.__root__)
-            property = scalars[0].__root__.property
-            value = scalars[1].__root__.__root__
-            query = Q(
-                {'match': {f'{property}': f'{value}'}}
-            )
-            q.must(query)
-    elif not getattr(node, 'intersects', 0) == 0:
-        property = node.intersects.__root__[0].__root__.property
-        if property == 'geometry':
-            geom_type = node.intersects.__root__[
-                1].__root__.__root__.__root__.type
-            if geom_type == 'Polygon':
-                coordinates = node.intersects.__root__[
-                    1].__root__.__root__.__root__.coordinates
-                coords_list = [
-                    poly_coords.__root__ for poly_coords in coordinates[0]
-                ]
-                filter_ = Q(
-                    {
-                        'geo_shape': {
-                            'geometry': {
-                                'shape': {
-                                    'type': 'envelope',
-                                    'coordinates': get_envelope(
-                                        coords_list)
-                                },
-                                'relation': 'intersects'
-                            }
-                        }
-                    }
-                )
-                query_all = Q(
-                    {'match_all': {}}
-                )
-                q.must(query_all)
-                q.filter(filter_)
-    return q.build()
-
-
-def update_query(input_query: Dict, cql: CQLModel):
+def update_query(input_query: Dict, cql):
     s = Search.from_dict(input_query)
-    query = OpenSearchQueryBuilder()
-    output_query = _build_query(query, cql)
-    s = s.query(output_query)
+    s = s.query(to_filter(cql))
 
     LOGGER.debug(f'Enhanced query: {json.dumps(s.to_dict())}')
     return s.to_dict()
