@@ -5,7 +5,7 @@
 #          Abdulazeez Abdulazeez Adeshina <youngestdev@gmail.com>
 #
 # Copyright (c) 2020 Francesco Bartoli
-# Copyright (c) 2024 Tom Kralidis
+# Copyright (c) 2025 Tom Kralidis
 # Copyright (c) 2022 Abdulazeez Abdulazeez Adeshina
 #
 # Permission is hereby granted, free of charge, to any person
@@ -50,6 +50,7 @@ from starlette.responses import (
 import uvicorn
 
 from pygeoapi.api import API, APIRequest, apply_gzip
+import pygeoapi.api as core_api
 import pygeoapi.api.coverages as coverages_api
 import pygeoapi.api.environmental_data_retrieval as edr_api
 import pygeoapi.api.itemtypes as itemtypes_api
@@ -69,6 +70,7 @@ if 'PYGEOAPI_OPENAPI' not in os.environ:
 OPENAPI = load_openapi_document()
 
 if CONFIG['server'].get('admin'):
+    import pygeoapi.admin as admin_api
     from pygeoapi.admin import Admin
 
 p = Path(__file__)
@@ -102,28 +104,6 @@ def call_api_threadsafe(
     return api_call(*args)
 
 
-async def get_response(
-        api_call,
-        *args,
-) -> Union[Response, JSONResponse, HTMLResponse]:
-    """
-    Creates a Starlette Response object and updates matching headers.
-
-    Runs the core api handler in a separate thread in order to avoid
-    blocking the main event loop.
-
-    :param result: The result of the API call.
-                   This should be a tuple of (headers, status, content).
-
-    :returns: A Response instance.
-    """
-
-    loop = asyncio.get_running_loop()
-    headers, status, content = await loop.run_in_executor(
-        None, call_api_threadsafe, loop, api_call, *args)
-    return _to_response(headers, status, content)
-
-
 def _to_response(headers, status, content):
     if headers['Content-Type'] == 'text/html':
         response = HTMLResponse(content=content, status_code=status)
@@ -139,17 +119,20 @@ def _to_response(headers, status, content):
 
 
 async def execute_from_starlette(api_function, request: Request, *args,
-                                 skip_valid_check=False) -> Response:
-    api_request = await APIRequest.from_starlette(request, api_.locales)
+                                 skip_valid_check=False,
+                                 alternative_api=None
+                                 ) -> Response:
+    actual_api = api_ if alternative_api is None else alternative_api
+    api_request = await APIRequest.from_starlette(request, actual_api.locales)
     content: Union[str, bytes]
     if not skip_valid_check and not api_request.is_valid():
-        headers, status, content = api_.get_format_exception(api_request)
+        headers, status, content = actual_api.get_format_exception(api_request)
     else:
 
         loop = asyncio.get_running_loop()
         headers, status, content = await loop.run_in_executor(
             None, call_api_threadsafe, loop, api_function,
-            api_, api_request, *args)
+            actual_api, api_request, *args)
         # NOTE: that gzip currently doesn't work in starlette
         #       https://github.com/geopython/pygeoapi/issues/1591
         content = apply_gzip(headers, content)
@@ -167,7 +150,7 @@ async def landing_page(request: Request):
 
     :returns: Starlette HTTP Response
     """
-    return await get_response(api_.landing_page, request)
+    return await execute_from_starlette(core_api.landing_page, request)
 
 
 async def openapi(request: Request):
@@ -178,7 +161,7 @@ async def openapi(request: Request):
 
     :returns: Starlette HTTP Response
     """
-    return await get_response(api_.openapi_, request)
+    return await execute_from_starlette(core_api.openapi_, request)
 
 
 async def conformance(request: Request):
@@ -189,7 +172,7 @@ async def conformance(request: Request):
 
     :returns: Starlette HTTP Response
     """
-    return await get_response(api_.conformance, request)
+    return await execute_from_starlette(core_api.conformance, request)
 
 
 async def get_tilematrix_set(request: Request, tileMatrixSetId=None):
@@ -228,8 +211,8 @@ async def collection_schema(request: Request, collection_id=None):
     if 'collection_id' in request.path_params:
         collection_id = request.path_params['collection_id']
 
-    return await get_response(api_.get_collection_schema, request,
-                              collection_id)
+    return await execute_from_starlette(core_api.get_collection_schema,
+                                        request, collection_id)
 
 
 async def collection_queryables(request: Request, collection_id=None):
@@ -334,11 +317,7 @@ async def collection_items(request: Request, collection_id=None, item_id=None):
     if 'item_id' in request.path_params:
         item_id = request.path_params['item_id']
     if item_id is None:
-        if request.method == 'GET':  # list items
-            return await execute_from_starlette(
-                itemtypes_api.get_collection_items, request, collection_id,
-                skip_valid_check=True)
-        elif request.method == 'POST':  # filter or manage items
+        if request.method == 'POST':  # filter or manage items
             content_type = request.headers.get('content-type')
             if content_type is not None:
                 if content_type == 'application/geo+json':
@@ -347,7 +326,7 @@ async def collection_items(request: Request, collection_id=None, item_id=None):
                         'create', collection_id, skip_valid_check=True)
                 else:
                     return await execute_from_starlette(
-                        itemtypes_api.post_collection_items,
+                        itemtypes_api.get_collection_items,
                         request,
                         collection_id,
                         skip_valid_check=True,
@@ -357,6 +336,10 @@ async def collection_items(request: Request, collection_id=None, item_id=None):
                 itemtypes_api.manage_collection_item, request,
                 'options', collection_id, skip_valid_check=True,
             )
+        else:  # GET: list items
+            return await execute_from_starlette(
+                itemtypes_api.get_collection_items, request, collection_id,
+                skip_valid_check=True)
 
     elif request.method == 'DELETE':
         return await execute_from_starlette(
@@ -395,7 +378,7 @@ async def collection_coverage(request: Request, collection_id=None):
         skip_valid_check=True)
 
 
-async def collection_map(request: Request, collection_id, style_id=None):
+async def collection_map(request: Request, collection_id=None, style_id=None):
     """
     OGC API - Maps map render endpoint
 
@@ -489,34 +472,13 @@ async def get_job_result(request: Request, job_id=None):
                                         request, job_id)
 
 
-async def get_job_result_resource(request: Request,
-                                  job_id=None, resource=None):
-    """
-    OGC API - Processes job result resource endpoint
-
-    :param request: Starlette Request instance
-    :param job_id: job identifier
-    :param resource: job resource
-
-    :returns: HTTP response
-    """
-
-    if 'job_id' in request.path_params:
-        job_id = request.path_params['job_id']
-    if 'resource' in request.path_params:
-        resource = request.path_params['resource']
-
-    # TODO: this api function currently doesn't exist
-    return await get_response(
-        api_.get_job_result_resource, request, job_id, resource)
-
-
-async def get_collection_edr_query(request: Request, collection_id=None, instance_id=None):  # noqa
+async def get_collection_edr_query(request: Request, collection_id=None, instance_id=None, location_id=None):  # noqa
     """
     OGC EDR API endpoints
 
     :param collection_id: collection identifier
     :param instance_id: instance identifier
+    :param location_id: location id of a /locations/<location_id> query
 
     :returns: HTTP response
     """
@@ -527,10 +489,23 @@ async def get_collection_edr_query(request: Request, collection_id=None, instanc
     if 'instance_id' in request.path_params:
         instance_id = request.path_params['instance_id']
 
-    query_type = request["path"].split('/')[-1]  # noqa
+    if (request.url.path.endswith('instances') or
+            (instance_id is not None and
+             request.url.path.endswith(instance_id))):
+        return await execute_from_starlette(
+            edr_api.get_collection_edr_instances, request, collection_id,
+            instance_id
+        )
+
+    if 'location_id' in request.path_params:
+        location_id = request.path_params['location_id']
+        query_type = 'locations'
+    else:
+        query_type = request['path'].split('/')[-1]
+
     return await execute_from_starlette(
         edr_api.get_collection_edr_query, request, collection_id,
-        instance_id, query_type,
+        instance_id, query_type, location_id,
         skip_valid_check=True,
     )
 
@@ -546,8 +521,9 @@ async def collections(request: Request, collection_id=None):
     """
     if 'collection_id' in request.path_params:
         collection_id = request.path_params['collection_id']
-    return await get_response(
-        api_.describe_collections, request, collection_id)
+
+    return await execute_from_starlette(core_api.describe_collections, request,
+                                        collection_id)
 
 
 async def stac_catalog_root(request: Request):
@@ -581,11 +557,14 @@ async def admin_config(request: Request):
     """
 
     if request.method == 'GET':
-        return await get_response(ADMIN.get_config, request)
+        return await execute_from_starlette(admin_api.get_config_, request,
+                                            alternative_api=ADMIN)
     elif request.method == 'PUT':
-        return await get_response(ADMIN.put_config, request)
+        return await execute_from_starlette(admin_api.put_config, request,
+                                            alternative_api=ADMIN)
     elif request.method == 'PATCH':
-        return await get_response(ADMIN.patch_config, request)
+        return await execute_from_starlette(admin_api.patch_config, request,
+                                            alternative_api=ADMIN)
 
 
 async def admin_config_resources(request: Request):
@@ -596,9 +575,11 @@ async def admin_config_resources(request: Request):
     """
 
     if request.method == 'GET':
-        return await get_response(ADMIN.get_resources, request)
+        return await execute_from_starlette(admin_api.get_resources, request,
+                                            alternative_api=ADMIN)
     elif request.method == 'POST':
-        return await get_response(ADMIN.put_resource, request)
+        return await execute_from_starlette(admin_api.put_resource, request,
+                                            alternative_api=ADMIN)
 
 
 async def admin_config_resource(request: Request, resource_id: str):
@@ -614,17 +595,21 @@ async def admin_config_resource(request: Request, resource_id: str):
         resource_id = request.path_params['resource_id']
 
     if request.method == 'GET':
-        return await get_response(
-            ADMIN.get_resource, request, resource_id)
+        return await execute_from_starlette(admin_api.get_resource, request,
+                                            resource_id,
+                                            alternative_api=ADMIN)
     elif request.method == 'PUT':
-        return await get_response(
-            ADMIN.put_resource, request, resource_id)
+        return await execute_from_starlette(admin_api.put_resource, request,
+                                            resource_id,
+                                            alternative_api=ADMIN)
     elif request.method == 'PATCH':
-        return await get_response(
-            ADMIN.patch_resource, request, resource_id)
+        return await execute_from_starlette(admin_api.patch_resource, request,
+                                            resource_id,
+                                            alternative_api=ADMIN)
     elif request.method == 'DELETE':
-        return await get_response(
-            ADMIN.delete_resource, request, resource_id)
+        return await execute_from_starlette(admin_api.delete_resource, request,
+                                            resource_id,
+                                            alternative_api=ADMIN)
 
 
 class ApiRulesMiddleware:
@@ -683,7 +668,6 @@ api_routes = [
     Route('/jobs/{job_id}', get_jobs, methods=['GET', 'DELETE']),
     Route('/processes/{process_id}/execution', execute_process_jobs, methods=['POST']),  # noqa
     Route('/jobs/{job_id}/results', get_job_result),
-    Route('/jobs/{job_id}/results/{resource}', get_job_result_resource),
     Route('/collections/{collection_id:path}/position', get_collection_edr_query),  # noqa
     Route('/collections/{collection_id:path}/area', get_collection_edr_query),
     Route('/collections/{collection_id:path}/cube', get_collection_edr_query),
@@ -692,6 +676,8 @@ api_routes = [
     Route('/collections/{collection_id:path}/corridor', get_collection_edr_query),  # noqa
     Route('/collections/{collection_id:path}/locations', get_collection_edr_query),  # noqa
     Route('/collections/{collection_id:path}/locations/{location_id}', get_collection_edr_query),  # noqa
+    Route('/collections/{collection_id:path}/instances', get_collection_edr_query),  # noqa
+    Route('/collections/{collection_id:path}/instances/{instance_id}', get_collection_edr_query),  # noqa
     Route('/collections/{collection_id:path}/instances/{instance_id}/position', get_collection_edr_query),  # noqa
     Route('/collections/{collection_id:path}/instances/{instance_id}/area', get_collection_edr_query),  # noqa
     Route('/collections/{collection_id:path}/instances/{instance_id}/cube', get_collection_edr_query),  # noqa
@@ -703,7 +689,7 @@ api_routes = [
     Route('/collections', collections),
     Route('/collections/{collection_id:path}', collections),
     Route('/stac', stac_catalog_root),
-    Route('/stac/{path:path}', stac_catalog_path),
+    Route('/stac/{path:path}', stac_catalog_path)
 ]
 
 admin_routes = [
