@@ -35,7 +35,7 @@ from filelock import FileLock
 import functools
 from functools import partial
 from dataclasses import dataclass
-from datetime import date, datetime, time
+from datetime import date, datetime, time, timezone
 from decimal import Decimal
 from enum import Enum
 import json
@@ -128,10 +128,10 @@ def dategetter(date_property: str, collection: dict) -> str:
 
     value = collection.get(date_property)
 
-    if value is None:
-        return None
-
-    return value.isoformat()
+    if value is None or isinstance(value, str):
+        return value
+    else:
+        return value.isoformat()
 
 
 def get_typed_value(value: str) -> Union[bool, float, int, str]:
@@ -302,6 +302,11 @@ def format_datetime(value: str, format_: str = DATETIME_FORMAT) -> str:
     return dateutil.parser.isoparse(value).strftime(format_)
 
 
+def get_current_datetime(tz: timezone = timezone.utc,
+                         format_: str = DATETIME_FORMAT) -> str:
+    return datetime.now(tz).strftime(format_)
+
+
 def file_modified_iso8601(filepath: Path) -> str:
     """
     Provide a file's ctime in ISO8601
@@ -429,12 +434,13 @@ def is_url(urlstring: str) -> bool:
         return False
 
 
-def render_j2_template(config: dict, template: Path,
+def render_j2_template(config: dict, tpl_config: dict, template: Path,
                        data: dict, locale_: str = None) -> str:
     """
     render Jinja2 template
 
     :param config: dict of configuration
+    :param tpl_config: dict of template configuration
     :param template: template (relative path)
     :param data: dict of data
     :param locale_: the requested output Locale
@@ -448,7 +454,7 @@ def render_j2_template(config: dict, template: Path,
     LOGGER.debug(f'Locale directory: {locale_dir}')
 
     try:
-        templates = config['server']['templates']['path']
+        templates = tpl_config['path']
         template_paths.insert(0, templates)
         LOGGER.debug(f'using custom templates: {templates}')
     except (KeyError, TypeError):
